#!/usr/bin/env python
import ffmpeg
import click
import os

@click.command()
@click.argument('infile')
@click.argument('outdir')
@click.option('--audio', help="Audio language or stream number")
@click.option('--hardcode', help="Hardcode target sub stream (language or stream number)")
@click.option('--overwrite/--no-overwrite', default=True)
def main(infile, outdir, hardcode=None, audio='eng', overwrite=True):
    outfile = os.path.join(outdir, os.path.splitext(os.path.split(infile)[1])[0]+'.mp4')
    if os.path.exists(outfile) and not overwrite:
        click.abort(0)
    probe = ffmpeg.probe(infile)

    audio_streams, sub_streams = [], []
    for stream in probe['streams']:
        if stream['codec_type'] == 'audio':
            audio_streams.append(stream)
        elif stream['codec_type'] == 'subtitle':
             sub_streams.append(stream)

    if audio.isdigit():
        audio_index = audio
    elif len(audio_streams) == 1:
        audio_index = str(audio_streams[0]['index'])
    else:
        audio_index = next(str(stream['index']) for stream in audio_streams if stream['tags'].get('language') == audio)

    if hardcode:
        if hardcode.isdigit():
            sub_stream = hardcode
        elif len(sub_streams) == 1:
            sub_stream = sub_streams[0]
        else:
            sub_stream = next(stream for stream in sub_streams if stream['tags'].get('language') == hardcode)

    raw = ffmpeg.input(infile)
    if hardcode:
        sub_index = sub_stream['index']
        if sub_stream['codec_name'] == 'ass':
            video = ffmpeg.filter(raw.video, 'subtitles', filename=infile, si=sub_streams.index(sub_stream))
        else:
            video = ffmpeg.overlay(raw.video, raw[str(sub_index)], eof_action='pass')
    else:
        video = raw.video

    ffmpeg.output(video, raw[audio_index], outfile, acodec='aac', vcodec='libx264', crf='18', preset='slow').run(overwrite_output=True)

if __name__ == '__main__':
    main()
